Package: poLCAParallel
Type: Package
Title: Polytomous Variable Latent Class Analysis Parallel
Version: 1.2.7
Authors@R: c(person(given = "Sherman",
                    family = "Lo",
                    role = c("aut", "cre"),
                    email = "s.lo@qmul.ac.uk"),
             person(given = "Drew",
                    family = "Linzer",
                    role = "aut",
                    email = "drew@votamatic.org"),
             person(given = "Jeffrey",
                    family = "Lewis.",
                    role = "aut",
                    email = "jblewis@ucla.edu"))
Depends: R (>= 4.1.2)
Imports: Rcpp (>= 1.0.7), scatterplot3d, MASS, parallel, poLCA
LinkingTo: Rcpp, RcppArmadillo (>= 0.12.4.1.0)
Description: A 'C++' reimplementation of 'poLCA' - latent class analysis and
    latent class regression models for polytomous outcome variables, also known
    as latent structure analysis. It attempts to reproduce results and be as
    similar as possible to the original code, while running faster, especially
    with multiple repetitions, by utilising multiple threads. Further reading is
    available on the Queen Mary, University of London, IT Services Research blog
    <https://blog.hpc.qmul.ac.uk/speeding_up_r_packages/>.
License: GPL-2
URL: https://github.com/QMUL/poLCAParallel
LazyLoad: yes
NeedsCompilation: yes
RoxygenNote: 7.3.3
Encoding: UTF-8
Suggests: testthat (>= 3.0.0), roxygen2, usethis
Config/testthat/edition: 3
SystemRequirements: C++20
Packaged: 2026-03-12 12:17:11 UTC; ubuntu
Author: Sherman Lo [aut, cre],
  Drew Linzer [aut],
  Jeffrey Lewis. [aut]
Maintainer: Sherman Lo <s.lo@qmul.ac.uk>
Repository: CRAN
Date/Publication: 2026-03-13 08:00:08 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 21:14:48 UTC; unix
Archs: poLCAParallel.so.dSYM
