# Generated by roxygen2: do not edit by hand

S3method(coef,poLCA)
S3method(plot,poLCA)
S3method(print,poLCA)
S3method(vcov,poLCA)
export(blrt)
export(poLCA)
export(poLCA.entropy)
export(poLCA.posterior)
export(poLCA.predcell)
export(poLCA.reorder)
export(poLCA.simdata)
export(poLCA.table)
export(poLCAParallel.goodnessfit)
export(poLCAParallel.se)
export(rmulti)
importFrom(Rcpp,sourceCpp)
useDynLib(poLCAParallel, .registration = TRUE)
